/***************************************************************************/
/*										*/
/*			  	PROXIM RANGELAN2 LOW LEVEL DRIVER		*/
/*										*/
/*		PROXIM, INC. CONFIDENTIAL AND PROPRIETARY.  This source is the	*/
/*		sole property of PROXIM, INC.  Reproduction or utilization of	*/
/*		this source in whole or in part is forbidden without the written*/
/*		consent of PROXIM, INC.						*/
/*										*/
/*										*/
/*				(c) Copyright PROXIM, INC. 1994			*/
/*					All Rights Reserved			*/
/*										*/
/***************************************************************************/


#include <asm/system.h>
#include <asm/io.h>
#include <linux/delay.h>
#include "lld.h"
#include "asm.h"

#define	COLOR		1
#define	MinCursor	80 * 27 * 2
#define	MaxCursor   	80 * 48 * 2

unsigned_16		Cursor = MinCursor;
unsigned_16		ScreenColumn;
#define INT_OFF 0
#define INT_ON 0x01
FLAGS		LLSIntState = INT_ON;

extern unsigned_8	LLDDebugFlag;
char *LLSColors[]={
	"BLACK",
	"BLUE",
	"GREEN",
	"CYAN",
	"RED",
	"MAGENTA",
	"BROWN",
	"WHITE",
	"GRAY",
	"BRIGHT_BLUE",
	"BRIGHT_GREEN",
	"BRIGHT_CYAN",
	"BRIGHT_RED",
	"BRIGHT_MAGENTA",
	"YELLOW",
	"BRIGHT_WHITE",
	"BLACK_BACK",
	"BLUE_BACK",
	"GREEN_BACK",
	"CYAN_BACK",
	"RED_BACK",
	"MAGENTA_BACK",
	"BROWN_BACK",
	"WHITE_BACK",
	"BLINK"
};


/***************************************************************************/
/*	_outp, _outw, _inp, _inpw                                          */
/*                                                                         */      
/*      in/out byte/word assembly commands used by the library.            */
/*      Mapped into the equivalent linux subroutines.                      */
/*                                                                         */
/***************************************************************************/

void _outp(ADDRESS_SIZE Reg, unsigned_8 Val)
{
	outb(Val, Reg);
}

void _outpw(ADDRESS_SIZE Reg, unsigned_16 Val)
{
	outw(Val, Reg);
}

unsigned_8 _inp(ADDRESS_SIZE Reg)
{
	return inb(Reg);
}

unsigned_16 _inpw(ADDRESS_SIZE Reg)
{
	return inw(Reg);
}


/***************************************************************************/
/*																									*/
/*	Delay (millitime)																			*/
/*																									*/
/*	INPUT:	millitime -	Number of milliseconds to delay. 						*/
/*																									*/
/*	OUTPUT:																						*/
/*																									*/
/*	DESCRIPTION: This routine calls a BIOS function to delay the required	*/
/*              number of milliseconds.												*/
/*																									*/
/***************************************************************************/

void Delay (unsigned_16 millitime)
{
	while (millitime--)
        	udelay(1000);
}


/***************************************************************************/
/*																									*/
/*	Delay_uS (microtime)																		*/
/*																									*/
/*	INPUT:	microtime -	Number of microseconds to delay. 				  		*/
/*																									*/
/*	OUTPUT:																						*/
/*																									*/
/*	DESCRIPTION: This routine calls a BIOS function to delay the required	*/
/*              required number of microseconds.									*/
/*																									*/
/***************************************************************************/

void Delay_uS (unsigned_16 microtime)
{
        udelay(microtime);
}


/***************************************************************************/
/*																									*/
/*	OutString (*string, ColorCode)					 									*/
/*																									*/
/*	INPUT:  string - The null terminated string of characters we want			*/
/*                  displayed.															*/
/*         ColorCode - The character's color.										*/
/*																									*/
/*	OUTPUT:																						*/
/*																									*/
/*	DESCRIPTION: This routine will print the character on the screen.			*/
/*																									*/
/***************************************************************************/

void OutString (char *string, unsigned_8 ColorCode)
{
	do {
		OutChar (*string, ColorCode);
	} while (*string++);
}

/***************************************************************************/
/*																									*/
/*	OutChar (Character, ColorCode)					 									*/
/*																									*/
/*	INPUT:  Character - The character we want displayed.							*/
/*         ColorCode - The character's color.										*/
/*																									*/
/*	OUTPUT:																						*/
/*																									*/
/*	DESCRIPTION: This routine will print the character on the screen.			*/
/*																									*/
/***************************************************************************/

void OutChar (unsigned_8 Character, unsigned_8 ColorCode)

{
	if (LLDDebugFlag == COLOR) {	
		printk("%c[%s][%s]", Character, LLSColors[ColorCode&0x0f],LLSColors[((ColorCode&0xf0)>>4)+0x10]);
/*
		print character to Cursor
		set color of character
		if (++Cursor == MaxCursor) Cursor = MinCursor;
		print a "*" at Cursor
*/
/*		__asm__	(
				"pushf\n\t"
				"push	es\n\t"
				"push	di\n\t"
				"push	ax\n\t"
				"push	bx\n\t"

				"cld\n\t"

				"movw	ax, $0xB800\n\t"
				"movw	es, ax\n\t"

				"pop	bx\n\t"
				"pop	ax\n\t"
				"pop	di\n\t"
				"pop	es\n\t"
				"popf\n\t"
		);  */

/*		__asm		pushf
		__asm		push	es
		__asm		push	di
		__asm		push	ax
		__asm		push	bx

		__asm		cld

		__asm		mov	ax, 0B800H
		__asm		mov	es, ax

		__asm		mov	di, Cursor
		__asm		mov	al, Character
		__asm		stosb
		__asm		mov	bl, ColorCode
		__asm		mov	es: byte ptr [di], bl
		__asm		inc	di
		__asm		cmp	di, MaxCursor
		__asm		jb		DontDecrement
		__asm		mov	di, MinCursor

	DontDecrement:
		__asm		mov	Cursor, di
		__asm		mov	es: byte ptr [di], '*'
		__asm		mov	es: byte ptr [di + 1], 0f4h

		__asm		pop	bx
		__asm		pop	ax
		__asm		pop	di
		__asm		pop	es
		__asm		popf
*/	}
}



/***************************************************************************/
/*																									*/
/*	Beep (Frequency)																			*/
/*																									*/
/*	INPUT:	Frequency	-	Frequency to beep at.									*/
/*																									*/
/*	OUTPUT:																						*/
/*																									*/
/*	DESCRIPTION: This routine will turn the speaker on at the specified		*/
/*              frequency.		  															*/
/*																									*/
/***************************************************************************/

void Beep (unsigned_16 Frequency)

{/* 	__asm		push	ax
	__asm		push	bx
	__asm		push	dx
	__asm		pushf
	__asm		cli
	__asm		mov	bx, Frequency
	__asm		mov	al, 0B6h
	__asm		out	043h, al
	__asm		xor	ax, ax
	__asm		cmp	bx, 20
	__asm		jb		Sound_On_1
	__asm		mov	dx, 012h
	__asm		mov	ax, 034DEh
	__asm		div	bx

	Sound_On_1:
	__asm		out	042h, al
	__asm		mov	al, ah
	__asm		push	ax
	__asm		in	al, 61h
	__asm		in	al, 61h
	__asm		in	al, 61h
	__asm		pop	ax
	__asm		out	042h, al
	__asm		in		al, 061h
	__asm		or		al, 3
	__asm		push	ax
	__asm		in	al, 61h
	__asm		in	al, 61h
	__asm		in	al, 61h
	__asm		in	al, 61h
	__asm		in	al, 61h
	__asm		in	al, 61h
	__asm		pop	ax
	__asm		out	061h, al
	__asm		popf
	__asm		pop	dx
	__asm		pop	bx
	__asm		pop	ax

	Delay (84);

	__asm		push 	ax
	__asm		pushf
	__asm		cli
	__asm		in	  	al, 061h
	__asm		and	al, 11111100b
	__asm		push	ax
	__asm		in	al, 61h
	__asm		in	al, 61h
	__asm		in	al, 61h
	__asm		in	al, 61h
	__asm		in	al, 61h
	__asm		in	al, 61h
	__asm		pop	ax
	__asm		out	061h, al
	__asm		popf
	__asm		pop	ax
*/}



/***************************************************************************/
/*																									*/
/*	PreserveFlag ()																			*/
/*																									*/
/*	INPUT:			 																			*/
/*																									*/
/*	OUTPUT:				Current flags														*/
/*																									*/
/* DESCRIPTION:		This routine returns the current processor flags.		*/
/*																									*/
/***************************************************************************/

FLAGS PreserveFlag (void)
{
	return (LLSIntState);

/*	when manipulating CPU interrupts:
	FLAGS	oldflag = 0;
        save_flags(oldflag);
	return (oldflag);
*/
}


/***************************************************************************/
/*																									*/
/*	RestoreFlag ()																				*/
/*																									*/
/*	INPUT:				Previously preserved flag										*/
/*																									*/
/*	OUTPUT:			 																			*/
/*																									*/
/* DESCRIPTION:		Restore the previously preserved processor flags.     */
/*																									*/
/***************************************************************************/

void RestoreFlag (FLAGS OrigFlag)
{
#ifndef NO_INT
	if(OrigFlag == INT_ON) {
		/* These next two lines should be atomic */
		if(!test_and_set_bit(0, (void*)&LLSIntState))
			enable_irq(LLDIntLine1);
	} else if (OrigFlag == INT_OFF) {
		/* These next two lines should be atomic */
		if(test_and_clear_bit(0, (void*)&LLSIntState))
			disable_irq(LLDIntLine1);
	}

/*	when manipulating CPU interrupts:
        restore_flags(OrigFlag);
*/
#endif /* NO_INT */
}


/***************************************************************************/
/*																									*/
/*	EnableInterrupt ()																		*/
/*																									*/
/*	INPUT:			 																			*/
/*																									*/
/*	OUTPUT: 			 																			*/
/*																									*/
/*	DESCRIPTION: This function enables the processor's interrupts.				*/
/*																									*/
/***************************************************************************/

void EnableInterrupt (void)
{
#ifndef NO_INT
	/* if already on, don't turn on again - linux doesn't like that */
	/* the "0" means manipulate bit 0 of LLSIntState. */
	/* These next two lines should be atomic */
	if(!test_and_set_bit(0, (void*)&LLSIntState))
		enable_irq(LLDIntLine1);

/*	when manipulating CPU interrupts:
	__sti();
*/
#endif /* NO_INT */
}


/***************************************************************************/
/*																									*/
/*	DisableInterrupt ()																		*/
/*																									*/
/*	INPUT:																						*/
/*																									*/
/*	OUTPUT:																						*/
/*																									*/
/*	DESCRIPTION: This function disables the processor's interrupts.			*/
/*																									*/
/***************************************************************************/
 
void DisableInterrupt (void)
{
#ifndef NO_INT
	/* if already off, don't turn off again - linux doesn't like that */
	/* the "0" means manipulate bit 0 of LLSIntState. */
	/* These next two lines should be atomic */
	if(test_and_clear_bit(0, (void*)&LLSIntState))
		disable_irq(LLDIntLine1);

/*	when manipulating CPU interrupts:
	__cli();
*/
#endif /* NO_INT */
}


/***************************************************************************/
/*																									*/
/*	OutHexL (HexLong, ColorCode)    														*/
/*																									*/
/*	INPUT:  The 32-bit hexadecimal number we want displayed.						*/
/*         The number's color.															*/
/*																									*/
/*	OUTPUT:																						*/
/*																									*/
/*	DESCRIPTION: This routine will print the Hex number to the screen			*/
/*																									*/
/***************************************************************************/

void OutHexL (unsigned_32 HexLong, unsigned_8 ColorCode)
{
	OutHex ((unsigned_8)(HexLong >> 24), ColorCode);
	OutHex ((unsigned_8)(HexLong >> 16), ColorCode);
	OutHex ((unsigned_8)(HexLong >>  8), ColorCode);
	OutHex ((unsigned_8)(HexLong >>  0), ColorCode);
}


/***************************************************************************/
/*																									*/
/*	OutHexW (HexWord, ColorCode)   														*/
/*																									*/
/*	INPUT:  The 16-bit hexadecimal number we want displayed.						*/
/*         The number's color.															*/
/*																									*/
/*	OUTPUT:																						*/
/*																									*/
/*	DESCRIPTION: This routine will print the Hex number to the screen			*/
/*																									*/
/***************************************************************************/

void OutHexW (unsigned_16 HexWord, unsigned_8 ColorCode)
{
	OutHex ((unsigned_8)(HexWord >>  8), ColorCode);
	OutHex ((unsigned_8)(HexWord >>  0), ColorCode);
}


/***************************************************************************/
/*																									*/
/*	OutHex (HexByte, ColorCode)  	 														*/
/*																									*/
/*	INPUT:  The 8-bit hexadecimal number we want displayed.		 				*/
/*         The number's color.															*/
/*																									*/
/*	OUTPUT:																						*/
/*																									*/
/*	DESCRIPTION: This routine will print the Hex number to the screen			*/
/*																									*/
/***************************************************************************/

void OutHex (unsigned_8 HexByte, unsigned_8 ColorCode)
{
	unsigned_8	Abyte;

	if (LLDDebugFlag == COLOR) {	
		Abyte = (HexByte & 0xf0)>>4;
		if (Abyte < 10) Abyte += '0';
		else Abyte += 'A'-10;
		OutChar (Abyte, ColorCode);

		Abyte = HexByte & 0x0f;
		if (Abyte < 10) Abyte += '0';
		else Abyte += 'A'-10;
		OutChar (Abyte, ColorCode);
	}
}
