/****************************************************************
                                                                *
	lls.h							*
                                                                *
*****************************************************************/


/****************************************************************
                                                                *
	Prototypes for linux-related functions			*
                                                                *
*****************************************************************/

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,3,0)
#define net_device device
#define net_device_stats enet_statistics
#endif

int 	init_module(void);
static int 	Prox_probe(struct net_device *dev);

static void 	Prox_interrupt(int irq, void *dev_id, struct pt_regs *regs);
void 		Prox_interrupt_bh(void);

static int 	Prox_open(struct net_device *dev);
static void 	Prox_poll(unsigned long d);
static int 	Prox_start_xmit(struct sk_buff *skb, struct net_device *dev);

static int 	Prox_close(struct net_device *dev);
void 	cleanup_module(void);

static int 	Prox_ioctl(struct net_device *dev, struct ifreq *rq, int cmd);
#ifdef PROX_DIAG
unsigned_16 	Prox_diag_ioctl(struct net_device *dev, int command, unsigned_8 *data);
#endif
static struct 	net_device_stats *Prox_get_stats(struct net_device *dev);
#ifdef WIRELESS_EXT
static struct	iw_statistics *Prox_get_wireless_stats(struct net_device *dev);
#endif /* WIRELESS_EXT */
static void	Prox_set_multicast_list(struct net_device *dev);

#ifdef USBCARD
static void *Proxusb_probe (struct usb_device *dev, unsigned int ifnum);
static void Proxusb_close (struct usb_device *dev, void *ptr);
#endif /* USBCARD */


/****************************************************************
                                                                *
	prototypes for standard linux ethernet net_device	*
                                                                *
*****************************************************************/

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,4,0)
static char	devicename[9] = { 0, };
#endif
static struct	net_device thisRangeLan = {
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,4,0)
			devicename, 
#else
			"",
#endif
			0, 0, 0, 0,
	 		0x0, 0,
	 		0, 0, 0, NULL, Prox_probe
};
/* structure for dev->priv */
struct 		net_local {	
#ifdef WIRELESS_EXT
			struct iw_statistics wstats;
#endif /* WIRELESS_EXT */
			struct net_device_stats stats;
			int attempted_tx_packets;
};
struct tq_struct LLSISR_Sched;


/****************************************************************

	definitions for usb functions

*****************************************************************/

#ifdef USBCARD
static struct usb_driver Proxusb_driver = {
        "proxmod",
        &Proxusb_probe,
        &Proxusb_close,
	{NULL, NULL},
	NULL,
	0
};
#endif /* USBCARD */


/****************************************************************

	definitions for the LLDPoll () timer routine		*

*****************************************************************/

#define POLL_FREQ (HZ/10)
#define LIB_HZ 100
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,3,0)
static struct 	timer_list poll_timer={NULL,NULL,POLL_FREQ,0,Prox_poll};
#else
static struct 	timer_list poll_timer={{NULL,NULL},POLL_FREQ,0,Prox_poll};
#endif
static int 	poll_on = 0;


/****************************************************************
                                                                *
	variables used in CLLD that we want to see		*
                                                                *
*****************************************************************/

extern ADDRESS_SIZE LLDIOAddress1;
extern unsigned_8 LLDIntLine1;
extern MEMORY_SIZE LLDMemoryAddress1;
extern unsigned_8 LLDOEM;
extern unsigned_8 LLDMicroISA;
extern unsigned_8 LLDPCMCIA;
extern unsigned_8 LLDOnePieceFlag;
extern unsigned_8 LLDFlag375;
extern unsigned_8 LLDCardConfig;
extern unsigned_8 LLDRL2CardConfig;
extern unsigned_8 LLDCardMode;
extern unsigned_8 LLDUSBCard;
extern unsigned_8 PCMCIACardInserted;

extern unsigned_8 LLDNodeAddress[];
extern unsigned_8 LLDPhysAddress[];
extern unsigned_8 LLDROMVersion[];
extern unsigned_8 LLDBootVersion[];
extern unsigned_8 LLDChannel;
extern unsigned_8 LLDSubChannel;
extern unsigned_8 LLDDomain;
extern unsigned_8 LLDSyncName[];
extern unsigned_8 LLDTransferMode;
extern unsigned_8 LLDSyncState;
extern unsigned_8 LLDMSTASyncName[];
extern unsigned_8 LLDMSTAAddr[];
extern unsigned_8 LLDMSTASyncChannel;
extern unsigned_8 LLDMSTASyncSubChannel;
extern unsigned_8 LLDMSTAFlag;
extern unsigned_8 LLDNodeType;
extern unsigned_8 HomeRF;
extern unsigned_8 LLDMSTA_NWID[];
extern unsigned_8 LLDAsciiSoftSecID[];
extern unsigned_8 LLDHRFAsciiSoftSecID[];
extern unsigned_8 LLDSoftSecIDNotSupported;
extern unsigned_8 LLDLearningState;
extern unsigned_8 InTeachModeFlag;
extern unsigned_8 LLDTxMode;
extern unsigned_8 LLDNeedReset;
extern unsigned_8 LLDNeedResetCnt;
extern unsigned_8 TxFlag;
extern unsigned_8 LLDPeerToPeerFlag;
extern unsigned_8 LLDRoamConfiguration;
extern unsigned_8 LLDInactivityTimeOut;
extern unsigned_8 LLDHopPeriod;
extern unsigned_8 LLDBFreq;
extern unsigned_8 NodeOverrideFlag;
extern unsigned_8 InResetFlag;
extern unsigned_16 LLDTicksToSniff;
extern unsigned_16 LLDIORange1;

extern unsigned_32 LLDOutSyncDrp;
extern unsigned_32 LLDOutSyncQDrp;
extern unsigned_32 LLDTimedQDrp;
extern unsigned_32 LLDFailureDrp;
extern unsigned_32 LLDNoReEntrantDrp;
extern unsigned_32 LLDSendDisabledDrp;
extern unsigned_8  DriverVersionString[];
extern unsigned_8  CLLDVersion[];
extern unsigned_8  LLDCurrentCountryCode;
extern unsigned_8  LLDCurrentFreqOffset;
extern unsigned_8  LLDWirelessProtocol;

extern unsigned_8  LLDCCVal1;
extern unsigned_8  LLDCCVal2;
#define MAXVAL 10
#define FIRMWARE_ERR 20
extern struct RSSITable {
	unsigned_8 numval;
	struct val {
		unsigned_8 dbm;
		unsigned_8 hex;
	} val[MAXVAL];
} CurrRSSITable;

/* some library functions we call */

int LLDRoamConfig (int RoamConfig);
int LLDMacOptimize (int MacOptimize);


/****************************************************************

	misc stuff

*****************************************************************/

#define MAXMAC 19  /* defined by data_string in proxnet */
unsigned_8 LLSRetCSSConfig;
unsigned_8 LLSNumMAC;
unsigned_8 LLSMACFlag;
unsigned_8 LLSPCIFlag=0;
unsigned_8 LLSMacOptimizeVal;
unsigned_8 LLDDebugFlag = CLEAR;
signed_16 outcount;
unsigned_16 LLSMemorySize1 = 0x4000;
ADDRESS_SIZE LLSIOAddress2 = 0;
ADDRESS_SIZE LLSIORange2 = 0x80;
unsigned_8 DownloadingFlash = 0;
unsigned_8 CardRunningFirmware = 0;
unsigned_8 LLSPCIDevice = 0;
unsigned_16 LLSmanfid;
unsigned_16 LLSpid;
/* this is used to prevent LLDIsr from running as a bottom half.  If
   set, LLDIsr runs in the context of the interrupt.  You shouldn't
   need to use this for ioctl calls since Prox_ioctl runs in the
   context of application space.  In this driver the flag is only
   used to catch a subtle init failure that causes the interrupt
   to be masked and then unregistered. */
unsigned_8 LLSNobh = CLEAR;
unsigned_8 *LLSWaitFlag;
struct wait_queue *WaitFlagSet_queue;


/****************************************************************

	structures and library functions we use for
	sending mac-layer pings, etc

*****************************************************************/

typedef struct LLSTxPktDescriptor
{	
	unsigned_8	LLDTxdriverWS[6];
	unsigned_16	LLDTxDataLength;					/* Length of total TX data		*/
	struct LLDTxFragmentList FAR	*LLDTxFragmentListPtr;			/* Ptr to list of fragments	*/
	unsigned_16	LLDImmediateDataLength;			/* Length of immediate data	*/
	unsigned_8	LLDImmediateData [30];	/* Immediate data for ping header*/
	struct LLSTxPktDescriptor		*Next;								/* Next free descriptor			*/
	struct	LLDTxFragmentList		LLDFragmentList;					/* For allocate memory 			*/
} LLSTxDescriptor;

struct MACList {
	struct MACList *Prev;
	unsigned_8 MAC[6];
	unsigned_8 RSSI;
	unsigned_32 TimeStamp;
} *TailMACList = NULL;


static LLSTxDescriptor LLDProxPkt = {{0},0,0,0,{0},0};
static unsigned_8 Buffer[1484];

unsigned_16 LLDSendProximPacket (LLSTxDescriptor *TxProxPktDescript, unsigned_8 TxBuffer [], 
			unsigned_32 Length, unsigned_8 Dest [], unsigned_8 PacketType, unsigned_16 Subtype);
unsigned_8 LLDSendSetSecurityID(signed_8 *secid);
unsigned_8 LLDSendSetNWID(signed_8 *secid);
unsigned_8 LLDSendSetSoftSecID(signed_8 *secid);
unsigned_8 LLDSendSetSoftNWID(signed_8 *secid);
void LLDMakeAPList(unsigned_8 domain);
void LLDStopAPSearch(unsigned_8 domain);
void LLDPrintAPList(struct MastersList list[]);
void LLDRawProcess (unsigned_8 Buffer[], unsigned int Length);
int GetCountryCode (void);
unsigned_16 GetRSSITable (void);


/****************************************************************
                                                                *
	Misc constants						*
                                                                *
*****************************************************************/

#define LLDMAXSEND 7

#define STATION 0
#define ALTERNATE 1
#define MASTER 2

#define RL2ISA 0 /* regular ISA */
#define RL2MISA 1 /* mini */
#define RL2UISA 2 /* micro, XR */
#define PROXPCCARD 3
#define SYMPNPISA 4 /* PnP ISA */
#define PROXPCI 5
#define PROXUSB 6
#define PROX633x 7 /* micro JR card */

#define PCCARD_VENDOR_ID_PROXIM 	0x0126
#define PCCARD_DEVICE_ID_PROXIM_7200	0x1058
#define PCCARD_DEVICE_ID_PROXIM_7400	0x1158
#define PCCARD_DEVICE_ID_PROXIM_4400	0x1258
#define PCCARD_DEVICE_ID_PROXIM_4430	0xffff
#define PCCARD_VENDOR_ID_INTEL	  	0x0089
#define PCCARD_DEVICE_ID_INTEL_1  	0xffff

#define USB_VENDOR_ID_PROXIM          	0x08c4
#define USB_DEVICE_ID_PROXIM_4230     	0x1701
#define USB_VENDOR_ID_INTEL	  	0x8086
#define USB_DEVICE_ID_INTEL_1	  	0x0001

#define PCI_VENDOR_ID_PROXIM           	0x14b7
#define PCI_DEVICE_ID_PROXIM_NONE    	0x0000
#define PCI_DEVICE_ID_PROXIM    	0x0001

#define PROXIM_MAX_DEV 1
#define INTERRUPT_REGISTER 0x4c
#define INTERRUPT_STATUS 0x04
#define INTERRUPT_CLEAR 0x400

#define	HOME_RF_AUTO	0x00
#define	RL2_AUTO	0x01
#define	HOME_RF_ONLY	0x02
#define	RL2_ONLY	0x03

#define	LEARNSTATERL2STATION		0x01
#define	LEARNSTATERL2MSTR	  		0x02
#define	LEARNSTATEHRF				0x03
#define	LEARNSTATEDUALMODE		0x04

#define	HOMERFONLY					0x01
#define	OPENAIRONLY					0x02
#define	DUALMODE						0x03


/****************************************************************
                                                                *
	Message strings						*
                                                                *
*****************************************************************/

char *error_codes[]={
	"Successful Initialization",
	"Error resetting the card - wrong io address",
	"Error registering the irq with the kernel - is it taken?",
   	"Error sending the initialize command - bad irq",
	"Error getting the ROM version",
	"Error getting the global address",
	"Error setting the local address",
	"Error setting the inactivity time out",
	"Error unknown",
	"Error setting security ID",
	"Error setting MAC optimize",
	"Error setting roaming parameters",
	"Error sending multicast command",
	"Error in card and socket services",
	"Error configuring socket - improper card seating",
	"Error configuring socket - power improperly supplied",
	"Error configuring socket - no PCMCIA bus ready signal",
	"Error with the PC Card memory window",
	"Error detecting a PC Card - wrong memory window",
	"Error starting Search and Continue",
	"Error getting f/w status - f/w may be corrupted",
	"Error getting card configuration",
	"Error in wireless protocol configuration - change wp"
};


/****************************************************************

	stuff that can be passed via insmod options
	there are a couple more in the PCCARD section
	below.

*****************************************************************/

int io = 0x270;
int irq = 11;
int CardType = CARDTYPE;
unsigned_8 wp = HOME_RF_AUTO;
int pci_card_number = 0;

#if LINUX_VERSION_CODE > KERNEL_VERSION(2,1,15)
MODULE_AUTHOR("Dave Koberstein <davek@komacke.com>");
MODULE_DESCRIPTION("Proxim RangeLAN2/Symphony/HomeRF ethernet driver");
MODULE_PARM(io, "i");
MODULE_PARM(irq, "i");
MODULE_PARM(wp, "i");
MODULE_PARM(CardType, "i");
MODULE_PARM(pci_card_number, "i");
#endif


/****************************************************************

	some version dependent stuff

*****************************************************************/

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,1,59)
#define DEV_FREE_SKB(skb) dev_kfree_skb(skb, FREE_WRITE);
#else
#define DEV_FREE_SKB(skb) dev_kfree_skb(skb);
#endif

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,1,23)
#define test_and_set_bit(val, addr) set_bit(val, addr)
#define copy_to_user memcpy_tofs
#define copy_from_user memcpy_fromfs
#endif


/****************************************************************

	some LLD structures for use by send routine

*****************************************************************/

#ifdef BUFFER
static struct LLS_Buffers {
	struct LLDTxPacketDescriptor txDesc;
	struct LLDTxFragmentList txFragList;
	struct sk_buff *save_skbuff_ptr;
	} bfr_list[LLDMAXSEND];
int	ring;
#else
static struct 	LLDTxPacketDescriptor txDesc;
/* save pointer for release in LLSSendComplete */
static struct 	sk_buff *saveptr;
static struct  LLDTxFragmentList txFragList;
#endif /* BUFFER */


/****************************************************************

	For debugging purposes

*****************************************************************/

/* for debugging */
#define ZONE0 0x01
#define ZONE1 0x02
#define ZONE2 0x04
#define ZONE3 0x08
#define ZONE4 0x10
#define ZONE5 0x20
#define ZONE6 0x40
#define ZONE7 0x80
#define RL2DEBUG(args...) if(rl2_debug) printk(args);
#define RL2DEBUG0(args...) if(rl2_debug&ZONE0) printk(args);
#define RL2DEBUG1(args...) if(rl2_debug&ZONE1) printk(args);
#define RL2DEBUG2(args...) if(rl2_debug&ZONE2) printk(args);
#define RL2DEBUG3(args...) if(rl2_debug&ZONE3) printk(args);
#define RL2DEBUG4(args...) if(rl2_debug&ZONE4) printk(args);
#define RL2DEBUG5(args...) if(rl2_debug&ZONE5) printk(args);
#define RL2DEBUG6(args...) if(rl2_debug&ZONE6) printk(args);
#define RL2DEBUG7(args...) if(rl2_debug&ZONE7) printk(args);
int rl2_debug = CLEAR;


#ifdef PCCARD
/****************************************************************

	PC CARD declarations.

*****************************************************************/

/* Parameters that can be set with 'insmod' */
/* The old way: bit map of interrupts to choose from */
/* This means pick from 15, 14, 12, 11, 10, 9, 7, 5, 4, and 3 */
static u_int irq_mask = 0xdeb8;
/* Newer, simpler way of listing specific interrupts */
static int irq_list[4] = { -1 };

MODULE_PARM(irq_mask, "i");
MODULE_PARM(irq_list, "1-4i");

/*
   All the PCMCIA modules use PCMCIA_DEBUG to control debugging.  If
   you do not define PCMCIA_DEBUG at all, all the debug code will be
   left out.  If you compile with PCMCIA_DEBUG=0, the debug code will
   be present but disabled -- but it can then be enabled for specific
   modules at load time with a 'pc_debug=#' option to insmod.
*/
#ifdef PCMCIA_DEBUG
static int pc_debug = PCMCIA_DEBUG;
MODULE_PARM(pc_debug, "i");
#define DEBUG(n, args...) if (pc_debug>(n)) printk(KERN_DEBUG args);
static char *versionpc =
"adapted from dummy_cs.c 1.9 1998/07/16 23:44:06 (David Hinds)";
#else
#define DEBUG(n, args...)
#endif /* PCMCIA_DEBUG */

/*====================================================================*/

/*
   The event() function is this driver's Card Services event handler.
   It will be called by Card Services when an appropriate card status
   event is received.  The config() and release() entry points are
   used to configure or release a socket, in response to card
   insertion and ejection events.  They are invoked from the
   event handler. 
*/

static void ProxCSS_config(dev_link_t *link);
static void ProxCSS_release(u_long arg);
static int ProxCSS_event(event_t event, int priority,
		       event_callback_args_t *args);

/*
   The attach() and detach() entry points are used to create and destroy
   "instances" of the driver, where each instance represents everything
   needed to manage one actual PCMCIA card.
*/

static dev_link_t *ProxCSS_attach(void);
static void ProxCSS_detach(dev_link_t *);

/*
   You'll also need to prototype all the functions that will actually
   be used to talk to your device.  See 'pcmem_cs' for a good example
   of a fully self-sufficient driver; the other drivers rely more or
   less on other parts of the kernel.
*/

/*
   The dev_info variable is the "key" that is used to match up this
   device driver with appropriate cards, through the card configuration
   database.
*/

static dev_info_t dev_info = "rlmod";

/*
   A linked list of "instances" of the dummy device.  Each actual
   PCMCIA card corresponds to one device instance, and is described
   by one dev_link_t structure (defined in ds.h).

   You may not want to use a linked list for this -- for example, the
   memory card driver uses an array of dev_link_t pointers, where minor
   device numbers are used to derive the corresponding array index.
*/

static dev_link_t *dev_list = NULL;

/*
   A dev_link_t structure has fields for most things that are needed
   to keep track of a socket, but there will usually be some device
   specific information that also needs to be kept track of.  The
   'priv' pointer in a dev_link_t structure can be used to point to
   a device-specific private data structure, like this.

   A driver needs to provide a dev_node_t structure for each device
   on a card.  In some cases, there is only one device per card (for
   example, ethernet cards, modems).  In other cases, there may be
   many actual or logical devices (SCSI adapters, memory cards with
   multiple partitions).  The dev_node_t structures need to be kept
   in a linked list starting at the 'dev' field of a dev_link_t
   structure.  We allocate them in the card's private data structure,
   because they generally shouldn't be allocated dynamically.

   In this case, we also provide a flag to indicate if a device is
   "stopped" due to a power management event, or card ejection.  The
   device IO routines can use a flag like this to throttle IO to a
   card that is not ready to accept it.
*/
   
typedef struct local_info_t {
    dev_node_t	node;
    int		stop;
} local_info_t;
#endif /* PCCARD */
