/****************************************************************
                                                                *
        This file is just the front-end application for		*
	configuring the driver.  Nothing too special.		*
                                                                *
*****************************************************************/

#include <sys/ioctl.h>
#include <sys/types.h>
#include <linux/if.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
/* can't figure this out - including socket.h gives me errors */
extern int socket __P ((int __domain, int __type, int __protocol));

#include "lld.h"
#include "proxcfg.h"

#define SPACE printf("  ")
#define TAB printf("\t")
#define RETURN printf("\n")

struct Prox_ioctl_info *ioc;
struct ifreq ifr;
unsigned_8 CardMode;
void PrintUsage(void);
void atomac(unsigned_8 * string, unsigned_8 * mac);

int main(int argc, signed_8 *argv[])
{
	unsigned_16 i,j,k,l,m;
	signed_16 s;
	unsigned_8 data_string[20*15]; /* set to (MAXMAC+1) * 15 */
	unsigned_8 data_val, save_val;

	ioc = (struct Prox_ioctl_info *) &ifr.ifr_data;

/**************************************************************** 
   Print usage if called for
****************************************************************/   
	if(argc == 1 || !strcmp(argv[1], "--help")) {
		PrintUsage();
		exit(0);
	}
	
/**************************************************************** 
   Open a socket to the driver
****************************************************************/   
	s=socket(AF_INET, SOCK_DGRAM, 0);
	if(s==-1) {
	  perror("proxnet: socket");
	  exit(1);
	}

/**************************************************************** 
   Check for some default stuff.  We take eth0 as a default.
****************************************************************/   
	if(!strcmp(argv[1],"dev")) {
    	  if(argc >= 3) {
	    strcpy(ifr.ifr_name,argv[2]);
	    if(argc == 3) PrintUsage();
	    i=3;
	  } else {
  	    strcpy(ifr.ifr_name, "eth0");
	    i=2;
	    PrintUsage();
	  }
	} else {
  	   strcpy(ifr.ifr_name, argv[1]);
	    if(argc == 2) PrintUsage();
	   i=2;
	}

/**************************************************************** 
   This parses the arguments.
****************************************************************/   
	for(;i<argc;i++) {
	  switch (argv[i][0]) {
	    case 'p':
		/* ping a MAC at the driver level */
		if(!strcmp(argv[i],"ping")) {
		  if (++i >= argc) {
		  	PrintUsage();
		  	exit(1); }
		  ioc->cmd = SEND_PROX_PING;
		  ioc->data = data_string;
		  atomac(argv[i], data_string);
		  if(ioctl(s,SIOCDEVPRIVATE,&ifr) == -1) {
		    perror("proxnet: ioctl");
		    PrintUsage();
		    exit(1);
		  }
		  sleep(1);
		  ioc->cmd = GET_PROX_PING;
		  ioc->data = data_string;
		  if(ioctl(s,SIOCDEVPRIVATE,&ifr) == -1) {
		    perror("proxnet: ioctl");
		    PrintUsage();
		    exit(1);
		  }
		  printf("Ping responses from:\n");
		  for(j=0;j<data_string[0];j++) {
		    for(k=1; k<6; k++)
		  	printf("%2.2x:", data_string[j*7+k]);
		  	printf("%2.2x, ", data_string[j*7+k]);
			if ( data_string[j*7+k+1] != 0xff)
				printf("%d%%\n", data_string[j*7+k+1]);
			else printf("n/a\n");
		  }
		} else {
		  PrintUsage();
		  exit(1);
		}
		break;
	    case 'f':
		/* ping to broadcast and see who's there */
		if(!strcmp(argv[i],"find")) {
		  if (++i >= argc) {
		  	PrintUsage();
		  	exit(1); 
		  }
		  /* clear out any old entries */
		  ioc->cmd = GET_PROX_PING;
		  ioc->data = data_string;
		  if(ioctl(s,SIOCDEVPRIVATE,&ifr) == -1) {
		      perror("proxnet: ioctl");
		      PrintUsage();
		      exit(1);
		  }
		  printf("Dozing devices won't show.\n");
		  for (j=0; j<atoi(argv[i]); j++) {
		    ioc->cmd = SEND_PROX_PING;
		    ioc->data = data_string;
		    atomac("ffffffffffff", data_string);
		    if(ioctl(s,SIOCDEVPRIVATE,&ifr) == -1) {
		      perror("proxnet: ioctl");
		      PrintUsage();
		      exit(1);
		    }
		    printf("Ping responses from:\n");
		    sleep(1);
		    ioc->cmd = GET_PROX_PING;
		    ioc->data = data_string; ioctl(s,SIOCDEVPRIVATE,&ifr);
		    for(m=0;m<data_string[0];m++) {
		      for(k=1; k<6; k++)
		  	printf("%2.2x:", data_string[m*7+k]);
		  	printf("%2.2x, ", data_string[m*7+k]);
			if (data_string[m*7+k+1] != 0xff)
				printf("%d%%\n", data_string[m*7+k+1]);
			else printf("n/a\n");
		    }
		  }
		} else {
		  PrintUsage();
		  exit(1);
		}
		break;
	    case 'm':
		/* search for any masters */
		if(!strcmp(argv[i],"masters")) {
		  struct MastersList currlist[10];  /* per MAXROAMHISTLIST */
		  printf("Searching for masters ... updates every 5 seconds:\n");
		  if (++i >= argc) {
		  	PrintUsage();
		  	exit(1); }
	  	  ioc->data = (unsigned char *) &data_val;
	  	  ioc->cmd = GET_NODETYPE;
		  if(ioctl(s,SIOCDEVPRIVATE,&ifr) == -1) {
		    perror("proxnet: ioctl");
		    PrintUsage();
		    exit(1);
		  }
		  save_val = data_val;
	  	  data_val = 0; /* station */
	  	  ioc->cmd = SET_NODETYPE; ioctl(s,SIOCDEVPRIVATE,&ifr); 
	          ioc->cmd = SET_LLDRESET; ioctl(s,SIOCDEVPRIVATE,&ifr);
		  sleep(1); /* to let reset take affect */
		  ioc->cmd = START_AP_SEARCH;
		  ioc->data = data_string;
		  if(ioctl(s,SIOCDEVPRIVATE,&ifr) == -1) {
		    perror("net: ioctl");
		    PrintUsage();
		    exit(1);
		  }
		  for (k=0; k<(atoi(argv[i])/5); k++) {
		    sleep(5);
		    ioc->data = (unsigned_8 *) currlist;
		    ioc->cmd = AP_LIST; ioctl(s,SIOCDEVPRIVATE,&ifr);
		    printf("Time searched: %d seconds\n", (k+1)*5);
		    l=0;
		    while (currlist[l].ChSubChannel) {
		      printf("\tName:%s, MAC:", currlist[l].Name );
		      for(j=0;j<5;j++)
		  	  printf("%2.2x:", currlist[l].Address[j]);
		      printf("%2.2x", currlist[l].Address[j]);
		      printf(", Ch:%d, Sub:%d\n", (currlist[l].ChSubChannel>>4)&0xf, currlist[l].ChSubChannel&0xf);
		      l++;
		    }
		    if (!l)
		      printf("No masters found.\n");
		  }
		  ioc->cmd = STOP_AP_SEARCH; ioctl(s,SIOCDEVPRIVATE,&ifr);
		  data_val = save_val;
	  	  ioc->data = (unsigned char *) &data_val; 
		  ioc->cmd = SET_NODETYPE; ioctl(s,SIOCDEVPRIVATE,&ifr); 
	          ioc->cmd = SET_LLDRESET; ioctl(s,SIOCDEVPRIVATE,&ifr);
		} else {
		  PrintUsage();
		  exit(1);
		}
		break;
	    default:
		PrintUsage();
		exit(1);
	  }
	}
	return 0;
}


void PrintUsage(void)
{
	printf("Usage:\tproxnet\t--help\n");
	printf("\tproxnet\t<ifname>|<dev {ifname}> [ping <wireless mac address>]\n");
	printf("\t\t[find <seconds to find>] [masters <search time>]\n");
}


void atomac(unsigned_8 * ptr, unsigned_8 * mac)
{
		  unsigned_8 index, j, k, digit, *string;
		  string = ptr;
		  for (j=0; j<6; j++) {
		    mac[j]=0;
		    for(k=0;k<2;k++) {
			index=2*j+k;
			if       ((digit = *(string+index)-'0') < 10);
			else if (((digit = *(string+index)-'a'+10) >=10) && (digit <= 0xf));
			else if (((digit = *(string+index)-'A'+10) >=10) && (digit <= 0xf));
			else digit=0;
			mac[j] += ( (digit&=0x0f) << ((1-k)*4));
			if ( *(string+index+1) == ':' ) string++;
			if ( *(string+index+1) == '-' ) string++;
			if ( *(string+index+1) == ' ' ) string++;
		    }
		  }
}
